function [T, N, U] = solve_pulsed(t0, tF, a, K, N0, u_pulse, del_t_on, del_t_off)
% Function to solve N' = aN(1-N/K) - u(t)*N for pulsed control u(t)
% Note: logistic growth and log-kill hypothesis
% Arguments:
    % t0:  initial time
    % tF:  final time
    % a:  growth rate
    % K:  carrying capacity
    % N0:  initial tumor size
    % u_pulse:  value of u(t) when therapy is being applied
                % implicitly u=0 otherwise
    % del_t_on:  length of treatment window
    % del_t_off:  rest period between treatments
% Returns:
    % T:  vector of times solution is evalued on
    % N:  corresponding population size
    % U:  vector of applied treatment
    
% First get how many treatment rounds to apply
% That is, full cycles (1 cycle:  u= u_pulse for del_t_on, followed by u=0
    % for del_t_off
cycle_length = del_t_on + del_t_off;
num_cycles = floor(tF/cycle_length);  % Full number of on-off cycles
t_remain = tF - num_cycles*cycle_length;  % Remaining time
t_pulse_start = t0 + cycle_length*(0:1:num_cycles-1);  % Times when pulses start (stay on for del_t_on)

% Now solve for the full num_cycles
% Initialize vectors to hold data
T = [t0];
N = [N0];
U = [u_pulse];  % Since starting on (with u_pulse) first element is u_pulse (and not 0)

% Now look through num_cyles of on followed by off
for i=1:num_cycles
    t_on = t_pulse_start(i);
    t_off = t_on + del_t_on;
    t_end_cycle = t_off + del_t_off;
    % First solve for the pulse and append
    [T_on,N_on] = ode23(@(t,N)rhs(t,N,a,K,u_pulse), [t_on t_off], N(end));
    T = [T;T_on];
    N = [N;N_on];
    U = [U;u_pulse*ones(length(T_on),1)];
    
    % Now solve during the rest period (u=0)
    [T_off,N_off] = ode23(@(t,N)rhs(t,N,a,K,0), [t_off t_end_cycle], N(end));
    T = [T;T_off];
    N = [N;N_off];
    U = [U;0*ones(length(T_off),1)];
    
end

% Lastly, account for any remaining time not covered by a complete cycle
if t_remain <= del_t_on && t_remain > 0
    % Just have to add an on pulse
    [T_on,N_on] = ode23(@(t,N)rhs(t,N,a,K,u_pulse), [T(end) tF], N(end));
    T = [T;T_on];
    N = [N;N_on];
    U = [U;u_pulse*ones(length(T_on),1)];
elseif t_remain > 0
    % Have to add both on and off pulse
    t_left_on = T(end) + del_t_on;
    [T_on,N_on] = ode23(@(t,N)rhs(t,N,a,K,u_pulse), [T(end) t_left_on], N(end));
    T = [T;T_on];
    N = [N;N_on];
    U = [U;u_pulse*ones(length(T_on),1)];
    
    [T_off,N_off] = ode23(@(t,N)rhs(t,N,a,K,0), [t_left_on tF], N(end));
    T = [T;T_off];
    N = [N;N_off];
    U = [U;0*ones(length(T_off),1)];
end

end

function dNdt = rhs(t, N, a, K, u)
% Auxillary vector field for logisitc growth
% Used in above function

dNdt = a*N.*(1-N/K) - u*N;

end

